<!--//
////////////////////////////////////////////////////////////////////////////////
//
//  Copyright  2009-2012 Nick Connors
//	gpumonitor@gmail.com
//
////////////////////////////////////////////////////////////////////////////////
//-->
System.Gadget.settingsUI = "settings.html";
System.Gadget.onSettingsClosed = onSettingsClosed;
System.Gadget.onShowSettings = onSettingsOpen;
System.Gadget.Flyout.file = "flyout.html";
System.Gadget.onUndock = checkDockedState;
System.Gadget.onDock = checkDockedState;

var redrawAll = true;
var resetMinMax = false;
var myTimer;
var sideshowContent;
var monitorCount = 0;
var monitorCountMismatch = false;

function onLoad()
{
	debug = false;
	getEnvVariables();
	if(debug)createDebugLog("Gadget version: " + version);
	if(!fso.FolderExists(userDataDir))fso.CreateFolder(userDataDir);
	getPrefIndex();
	launchHelper();
	loadSettings();
	setTitle();
	loadMonitorPrefs();
	resetMonitorPaths();
	resetMinMaxValues();
	checkDockedState();
    myTimer = setInterval(onTimer, prefWindow.refreshInterval*1000);
}

function onUnload()
{
	stopHelper();
	if(!savePrefMonitors()){
		pause(3000);
	}
	
	if(debug){
		writeDebug("Closing Gadget");
		debugFile.Close();
	}
}

function checkDockedState()
{
	if(System.Gadget.docked){
		document.body.style.width = "130px";
		if(prefWindow.bgColor == 'transparent'){
			document.body.style.fontFamily = 'Tahoma';//Arial';
		}else{
			document.body.style.fontFamily = 'Segoe UI';
		}
		document.getElementById('ss').href = 'css/shared.css';
		barWidths = barWidthsSm;
		divHeight = divHeightSm;
		document.getElementById('errMsg').style.height = '92px';
	}else{
		document.body.style.width = '200px';
		document.body.style.fontFamily = 'Segoe UI';
		document.getElementById('ss').href = 'css/shared-floating.css';
		barWidths = barWidthsLg;
		divHeight = divHeightLg; 
		document.getElementById('errMsg').style.height = '84px';
	}
	redrawAll = true;
	onTimer();
}

function launchHelper()
{
	if(prefIndex != 0) return;
	var objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\\\.\\root\\cimv2");     
    var processes = objWMIService.ExecQuery("Select * from Win32_Process Where Name = 'HWiNFOMonitor.exe'");
	
	if(debug)writeDebug("Checking for existing HWiNFOMonitor.exe processes - " + processes.Count + " found.");
	if(processes.Count == 0){
		try{
			var intMonitorProcessID;
			var objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\\\.\\root\\cimv2:Win32_Process");
			var error = objWMIService.Create(appDir + "\\HWiNFOMonitor.exe", null, null, intMonitorProcessID);
			if(debug)writeDebug("Launch HWiNFOMonitor.exe processes return code - " + error);
		}catch(ex){
			displayError("Could not launch HWiNFOMonitor.exe", ex);
		}
	}
}

function stopHelper()
{
	if(prefIndex != 0) return;
	var objWMIService = GetObject("winmgmts:{impersonationLevel=impersonate}!\\\\.\\root\\cimv2");     
    var processes = objWMIService.ExecQuery("Select * from Win32_Process Where Name = 'HWiNFOMonitor.exe'");
	
	if(debug)writeDebug("Stopping existing HWiNFOMonitor.exe processes - " + processes.Count + " found.");
	var e = new Enumerator(processes);
	for (; !e.atEnd(); e.moveNext())
	{
		e.item().Terminate();
	}
}

function loadMonitorPrefs()
{
	var tmpFile = userDataDir + "\\prefMonitors_" + prefIndex + ".json";
	if(fso.FileExists(tmpFile))
	{
		for(i = 1; i <= 3; i++){
			try{
				if(debug)writeDebug("Reading monitor settings: " + tmpFile);
				var stream = fso.OpenTextFile(tmpFile,1);
				var fileMonitorPreferences = stream.ReadAll();
				stream.Close();
				prefMonitors = jsonParse(fileMonitorPreferences);
				if(upgradeVersion){upgradeMonitors()}
				break;
			}catch (ex){
				if(i == 3){
					displayError("Could not read monitor preferences after " + i + " attempts - restoring default settings", ex);
				}else{
					displayError("Could not read monitor preferences after " + i + " attempts", ex);
				}
			}
			pause(750);			
		}
	}
}

function onSettingsClosed(p_event)
{
	if (p_event.closeAction == p_event.Action.commit) {
		if(debug)writeDebug("Settings closed commiting changes");
        getPrefIndex();
		redrawAll = true;
		loadSettings();
		loadMonitorPrefs();
		resetMonitorPaths();
		resetMinMaxValues();
		setTitle();
		setInitialHeight();
		checkDockedState();
		if(System.Gadget.Settings.readString("error") > "") displayError(System.Gadget.Settings.readString("error"));
    }else{
		if(debug)writeDebug("Settings closed cancelling changes");
	}
	onTimer();
	myTimer = setInterval(onTimer, prefWindow.refreshInterval*1000);
}

function onSettingsOpen()
{
	if(debug)writeDebug("Opening settings");
	clearInterval(myTimer);
	for(i = 0; i < 4; i++){
		if(rtMonitors.errorCode == 1){
			displayError("Waiting for HWiNFO's memory to reinitialize");
			pause(1000);
			onTimer();
		}else{
			break;
		}
	}
	savePrefMonitors();
}

function onTimer()
{
	sideshowContent = "";
	
	if(parseRTMonitors()){
		try{
        	if(rtMonitors.error > ""){
				displayError(rtMonitors.error);
				return false;
			}else{
				document.getElementById("errMsg").style.display = "none";
				if(redrawAll) {
					document.getElementById("monitorContainer").innerHTML = "";
					document.getElementById("monitorContainer").style.display = 'block';
				}
				previousReadWriteCollision = 0;
				if(drawMonitors()) {
					document.body.style.height = prefWindow.height + "px";
				}
			}
    	}catch(ex){
			displayError("Could not render monitor", ex);
		}		
	}
		
	if (sideshowEnabled){
		System.Gadget.SideShow.remove(1);
		
		if(prefWindow.dspTitle){
			System.Gadget.SideShow.addText(1, "<body><menu id='1' title='" + document.getElementById("divModel").innerText + "'>" + sideshowContent + "</menu></body>");
		}else{
			System.Gadget.SideShow.addText(1, "<body><menu id='1' title='HWiNFO Sensor Status'>" + sideshowContent + "</menu></body>");
		}
	}
}

function resetMinMaxValues(){
	for (var i = 0; i < prefMonitors.length; i++){
		prefMonitors[i].minval = prefMonitors[i].modValue;
		prefMonitors[i].maxval = prefMonitors[i].modValue;
	}
}

function resetMonitorPaths() {
	for (var i = 0; i < prefMonitors.length; i++){
		prefMonitors[i].path = [];
		if(getArrayIndex(graphStyles,prefMonitors[i].style) > -1){
			for(var j = 0; j < 100; j++){
				prefMonitors[i].path[j] = -1;
			}
		}
	}
}

function setInitialHeight() {
	if(debug)writeDebug("Setting initial gadget height");
	if(!prefWindow.titleEnabled){
		prefWindow.height = 7;
	}else{
		if(prefWindow.hasTitleBorder){
			prefWindow.height = 43;
		}else{
			prefWindow.height = 37;
		}
		if(!System.Gadget.docked){
			prefWindow.height += 8;
		}
	}	
}

function getMonitorPref(rtMon){
	for (var i = 0; i < prefMonitors.length; i++){
		if (prefMonitors[i].id == rtMon.id)return prefMonitors[i]
	}
	if(debug)writeDebug("Monitor: '" + rtMon.title + "' not found in preference list after searching " + prefMonitors.length + " monitors - creating new default monitor");
	//if(debug)writeDebug("Current rtMonitors are: " + JSON.stringify(rtMonitors));
	//if(debug)writeDebug("Current prefMonitors are: " + JSON.stringify(prefMonitors));
						
	var newMon = new DefaultMonitor();
	for(var i = 0; i < 100; i++){
		newMon.path[i] = 0;
	}
	
	var title = rtMon.title.toLowerCase();
	switch (rtMon.units.toUpperCase())
	{
		case "\u00b0C": { 
			newMon.iconIndex = 0;
			newMon.style = 17;
			newMon.barLimit = [20,75,80,85,90];
			if(title.indexOf("temperature")) newMon.dspTitle = rtMon.title.replaceAll("temperature","temp")
			break;
		}
		case "\u00b0F": { 
			newMon.iconIndex = 0;
			newMon.style = 17;
			newMon.barLimit = [68,167,176,185,194];
			break;
		}
		case "%": {			
			newMon.barLimit = [0,85,90,95,100];
			newMon.style = 17;
			if(title.indexOf("fan") != -1){
				newMon.iconIndex = 2;
			}else{
				newMon.iconIndex = 4;
			}
			break;
		}
		case "RPM": {
			newMon.style = 17;
			newMon.iconIndex = 2;
			newMon.barLimit = [0,3500,4000,4500,5000];
			break;
		}
		case "MHZ": {
			newMon.style = 17;
			newMon.iconIndex = 4;
			var tmpVal = trimValue(rtMon.value, rtMon.units);
			newMon.barLimit = [0,tmpVal+1,parseInt(tmpVal*1.1),parseInt(tmpVal*1.2),tmpVal*2];
			if(title.indexOf(" clock \\")) newMon.dspTitle = rtMon.title.replaceAll(" clock \\","")
			break;
		}
		case "FPS": {
			newMon.style = 17;
			newMon.iconIndex = 12;
			newMon.barLimit = [0,10,15,20,100];
			newMon.barColors = ["#FF0000", "#FF6600", "#FFFF00", "#36B74E"];
			break;
		}
		case "V": {
			newMon.style = 17;
			newMon.iconIndex = 6;
			var tmpVal = parseFloat(trimValue(rtMon.value, rtMon.units));
			newMon.barLimit = [tmpVal-0.2,tmpVal-0.1,tmpVal,tmpVal+0.1,tmpVal+0.2];
			newMon.barColors = ["#FF0000", "#36B74E", "#36B74E", "#FF0000"];
			break;
		}
		case "W": {
			newMon.style = 17;
			newMon.iconIndex = 24;
			var tmpVal = parseFloat(trimValue(rtMon.value, rtMon.units));
			newMon.barLimit = [tmpVal-0.2,tmpVal-0.1,tmpVal,tmpVal+0.1,tmpVal+0.2];
			newMon.barColors = ["#FF0000", "#36B74E", "#36B74E", "#FF0000"];
			break;
		}
		case "A": {
			newMon.style = 17;
			newMon.iconIndex = 6;
			var tmpVal = parseFloat(trimValue(rtMon.value, rtMon.units));
			newMon.barLimit = [tmpVal-2,tmpVal-1,tmpVal,tmpVal+1,tmpVal+2];
			newMon.barColors = ["#FF0000", "#36B74E", "#36B74E", "#FF0000"];
			break;
		}
		case "MB": {
			newMon.style = 17;
			newMon.iconIndex = 4;
			newMon.barColors = ["#36B74E", "#FFFF00", "#FF6600", "#FF0000"];
			switch (rtMon.title)
			{
				case "Physical Memory Used": {					
					try
					{
						var oWMI = GetObject("winmgmts:{impersonationLevel=impersonate}!//./root/cimv2");
						var cItems = oWMI.ExecQuery("Select * from Win32_ComputerSystem");
						var cItem = new Enumerator(cItems);
						for (; !cItem.atEnd(); cItem.moveNext())
						{
							var cProperties = new Enumerator(cItem.item().Properties_);
							for (; !cProperties.atEnd(); cProperties.moveNext())
							{
								if(cProperties.item().Name == "TotalPhysicalMemory") {
									var totalRAM = parseInt(cProperties.item().Value/1024/1024);
									newMon.barLimit = [0,parseInt(totalRAM*.85),parseInt(totalRAM*.9),parseInt(totalRAM*.95),totalRAM];
									break;
								}
							}
						}
					} catch (ex)
					{
						newMon.barLimit = [0,1740,1843,1945,2048];
					}
					break;
				}
				case "Virtual Memory Commited": {
					try
					{
						var oWMI = GetObject("winmgmts:{impersonationLevel=impersonate}!//./root/cimv2");
						var cItems = oWMI.ExecQuery("Select * from Win32_OperatingSystem");
						var cItem = new Enumerator(cItems);
						for (; !cItem.atEnd(); cItem.moveNext())
						{
							var cProperties = new Enumerator(cItem.item().Properties_);
							for (; !cProperties.atEnd(); cProperties.moveNext())
							{
								if(cProperties.item().Name == "TotalVisibleMemorySize") {var totalRAM = parseInt(cProperties.item().Value / 1024);}
								if(cProperties.item().Name == "TotalVirtualMemorySize") {var totalVirtual = parseInt(cProperties.item().Value / 1024);}
							}
							var pageFileSize = totalVirtual - totalRAM;
							newMon.barLimit = [0,parseInt(pageFileSize*.85),parseInt(pageFileSize*.9),parseInt(pageFileSize*.95),pageFileSize];
						}
					} catch (ex)
					{
						newMon.barLimit = [0,1740,1843,1945,2048];
					}
					break;
				}
				default: {
					newMon.barLimit = [0,70,80,90,100];
				}
			}
		}
		case "": {
		}
		default: {
		}
	}
	prefMonitors.push(newMon);
	return newMon; 
}

function drawMonitors()
{
	if (rtMonitors == undefined){
		if(debug)writeDebug("Can not draw any monitors - rtMonitors is undefined");
		return false;
	}
	try{
		if(monitorCount != rtMonitors.monitors.length) {
			if(monitorCountMismatch) {
				monitorCount = rtMonitors.monitors.length;
				monitorCountMismatch = false;
				redrawAll = true;
			} else {
				monitorCountMismatch = true;
			}
		} else {
			monitorCountMismatch = false;
		}
		if (sideshowEnabled || redrawAll) reorderMonitors();
		if(redrawAll) {
			document.getElementById("monitorContainer").innerHTML = "";
			document.getElementById("monitorContainer").style.display = 'block';
			setInitialHeight();			
		}
		for(var i = 0; i < rtMonitors.monitors.length; i++){
			var curMonitor;
			if(rtMonitors.monitors[i] == undefined){
				if(debug)writeDebug("Can not draw current monitor -  Monitor " + i + " is undefined");
				redrawAll = true;
				return false;
			}
			if(!rtMonitors.monitors[i]){
				if(debug)writeDebug("Can not draw current monitor - Monitor " + i + " is not a value");
				redrawAll = true;
				return false;
			}
			curMonitor = getMonitorPref(rtMonitors.monitors[i]);
			curMonitor.id = rtMonitors.monitors[i].id;
			curMonitor.title = rtMonitors.monitors[i].title;
			curMonitor.units = rtMonitors.monitors[i].units;
			curMonitor.value = rtMonitors.monitors[i].value;
			curMonitor.sensor = rtMonitors.monitors[i].sensor;
			if(curMonitor.units == "\u00b0C" && prefWindow.temperatureScale == 1){
				curMonitor.units = "\u00b0F";
				curMonitor.value = toFahrenheit(curMonitor.value);
			}
			translateValue(curMonitor);
			updateMinMax(curMonitor);
			if(curMonitor.enabled){			
				if(redrawAll){
					if(debug)writeDebug("Drawing Monitor: " + i + " - " + curMonitor.title);
					if(!curMonitor.dspTitle) curMonitor.dspTitle = curMonitor.title;									
					var div = document.createElement("div");
					div.setAttribute('id', curMonitor.id);
					document.getElementById("monitorContainer").appendChild(div);
					drawMonitor(curMonitor, div, i);
				}else{
					if(document.getElementById(curMonitor.id)){
						updateMonitor(curMonitor, document.getElementById(curMonitor.id), i);
					}
				}
			}
		}
		if(resetMinMax) resetMinMaxValues();
		redrawAll = false;
		monitorContainer.style.visibility = "";
		return true;
	}catch(ex){
		displayError("Error rendering monitor<br><br>" + rtMonitors.monitors[i].title, ex);
		redrawAll = true;
		return false;
	}	
}

function updateMinMax(monitor)
{
	if(monitor.modValue < monitor.minval){
		monitor.minval = monitor.modValue;
	}
	if(monitor.modValue > monitor.maxval){
		monitor.maxval = monitor.modValue;
	}
}

function loadSettings()
{
	var tmpFile = userDataDir + "\\prefWindow_" + prefIndex + ".json";
	if(fso.FileExists(tmpFile))
	{
		if(debug)writeDebug("Reading window settings: " + tmpFile);
		try{
        	var stream = fso.OpenTextFile(tmpFile, 1);
			var fileWindowPreferences = stream.ReadAll();
			stream.Close();
			prefWindow = jsonParse(fileWindowPreferences);
			if(prefWindow.version == undefined || prefWindow.version < version){upgradeSettings()}
    	}catch(ex){
			displayError("Could not parse window preferences", ex);
			return;
		}
	}else{
		if(debug)writeDebug("Window settings do not exist - creating default window");
		prefWindow = new DefaultWindow();
	}
	
	
	if(prefWindow.bgColor == "transparent"){
		//document.body.style.backgroundColor = "";
		document.getElementById("wrapper").className = "wrapper transparency";
		document.getElementById("wrapper").style.backgroundColor = "";
	}else{
		//document.body.style.backgroundColor = prefWindow.bgColor;	
		document.getElementById("wrapper").className = "wrapper";
		document.getElementById("wrapper").style.backgroundColor = prefWindow.bgColor;	
	}	
	
	//document.getElementById("background").style.backgroundColor = prefWindow.bgColor;
	if(prefWindow.borderColor == "transparent"){
		document.body.style.borderWidth = "0px";
		//document.body.style.padding = "2px";
		document.getElementById("background").style.padding = "2px";
	}else{
		document.body.style.borderWidth = "2px";
		document.body.style.borderColor = prefWindow.borderColor;
		//document.body.style.padding = "0px";
		document.getElementById("background").style.padding = "0px";
	}
	
	divModel.style.color = prefWindow.titleColor;
	if(prefWindow.SSEnabled){
		sideshowEnabled = true;
		initSideShow();
	}
}

function setTitle()
{
	if(debug)writeDebug("Setting title");

	if(prefWindow.titleEnabled){
		document.getElementById("divTitleContainer").style.display = 'block';
		divModel.innerHTML = prefWindow.dspTitle;
		if(prefWindow.titleIcon == 3){
			document.getElementById("divImgManufacturer").style.display = 'none';
		}else{
			document.getElementById("divImgManufacturer").style.display = 'block';
			imgManufacturer.src = icons[prefWindow.titleIcon][1];
		}
	}else{
		document.getElementById("divTitleContainer").style.display = 'none';
	}	
	
	if(prefWindow.hasTitleBorder){
		document.getElementById("divTitleOuter").style.borderStyle = "outset";
		document.getElementById("divTitleOuter").style.borderWidth = "2";
		document.getElementById("divTitleOuter").style.borderColor = prefWindow.titleBorderColor;
		document.getElementById("divTitleOuter").style.margin = "-1";
		document.getElementById("divTitleInner").style.borderStyle = "inset";
		document.getElementById("divTitleInner").style.borderWidth = "2";
		document.getElementById("divTitleInner").style.borderColor = prefWindow.titleBorderColor;
	}else{
		document.getElementById("divTitleOuter").style.borderStyle = "none";
		document.getElementById("divTitleInner").style.borderStyle = "none";
		document.getElementById("divTitleOuter").style.margin = "0";
	}	
}

function reorderMonitors() {
	var i;
	
	var requestedOrder = [];
	for(i = 0; i < prefMonitors.length; i++){
		requestedOrder.push(prefMonitors[i].id);
	}
	
	var sortedMonitors = new Array(rtMonitors.monitors.length);
	var newMonitors = [];
	
	for(i = 0; i < rtMonitors.monitors.length; i++){
		var index = getArrayIndex(requestedOrder, rtMonitors.monitors[i].id);
		if(index > -1)
			sortedMonitors[index] = rtMonitors.monitors[i];
		else
			newMonitors.push(rtMonitors.monitors[i]);
	}
	sortedMonitors.clean(undefined);
	rtMonitors.monitors = sortedMonitors.concat(newMonitors);	
}

function pause(millis) 
{
        var date = new Date();
        var curDate = null;

        do { curDate = new Date(); } 
        while(curDate-date < millis)
}

function displayError(error, ex)
{
	if(debug)writeDebug(error, ex);
	errMsg.innerHTML = "<center><strong>ERROR</strong></center>" + error;
	setInitialHeight();
	if(!prefWindow.titleEnabled){
		document.body.style.height = "112px";
	}else if(prefWindow.hasTitleBorder){
		document.body.style.height = "144px";
	}else{
		document.body.style.height = "138px";
	}	
	monitorContainer.style.display = 'none';
	errMsg.style.display = "block";
	redrawAll = true;
}
// SIG // Begin signature block
// SIG // MIIUewYJKoZIhvcNAQcCoIIUbDCCFGgCAQExCzAJBgUr
// SIG // DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
// SIG // gjcCAR4wJAIBAQQQEODJBs441BGiowAQS9NQkAIBAAIB
// SIG // AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFL4R0ExX4TQk
// SIG // G8gGtgkz+qB3RDQsoIIQRTCCA58wggKHoAMCAQICEHmi
// SIG // pYX50RVCE9m4Pva2je0wDQYJKoZIhvcNAQEFBQAwUzEL
// SIG // MAkGA1UEBhMCVVMxFzAVBgNVBAoTDlZlcmlTaWduLCBJ
// SIG // bmMuMSswKQYDVQQDEyJWZXJpU2lnbiBUaW1lIFN0YW1w
// SIG // aW5nIFNlcnZpY2VzIENBMB4XDTEyMDUwMTAwMDAwMFoX
// SIG // DTEyMTIzMTIzNTk1OVowYjELMAkGA1UEBhMCVVMxHTAb
// SIG // BgNVBAoTFFN5bWFudGVjIENvcnBvcmF0aW9uMTQwMgYD
// SIG // VQQDEytTeW1hbnRlYyBUaW1lIFN0YW1waW5nIFNlcnZp
// SIG // Y2VzIFNpZ25lciAtIEczMIGfMA0GCSqGSIb3DQEBAQUA
// SIG // A4GNADCBiQKBgQCpWWZ02j2KfXrY/PWARHv+R2oUVU5Q
// SIG // Rwvs0+3O9jj3T2m5sfC2eIIKjHYWZ+ICrbcNpYr2A/xm
// SIG // 0/wILcy1c1l7idwzbmZaXlI3tGLRklk1FItFrFmyTSSi
// SIG // mJRoQnKfOmjia4ueIi30mE6axq+z5KCrPCi/I+HXcqTy
// SIG // EFNnrnevUQIDAQABo4HjMIHgMAwGA1UdEwEB/wQCMAAw
// SIG // MwYDVR0fBCwwKjAooCagJIYiaHR0cDovL2NybC52ZXJp
// SIG // c2lnbi5jb20vdHNzLWNhLmNybDAWBgNVHSUBAf8EDDAK
// SIG // BggrBgEFBQcDCDA0BggrBgEFBQcBAQQoMCYwJAYIKwYB
// SIG // BQUHMAGGGGh0dHA6Ly9vY3NwLnZlcmlzaWduLmNvbTAO
// SIG // BgNVHQ8BAf8EBAMCB4AwHgYDVR0RBBcwFaQTMBExDzAN
// SIG // BgNVBAMTBlRTQTEtMzAdBgNVHQ4EFgQUtLfxiUkmYOdl
// SIG // 6nOu3NM4zb9Xkm8wDQYJKoZIhvcNAQEFBQADggEBAB6Y
// SIG // qie3eLUItclybbffwA6YpjXEiMnS9m3xSxr71fktmQCe
// SIG // 0eebi+E/vTmADGbNB7xcmFSmlLoQ0U6Lq/VvZcxnCaKA
// SIG // fFLoDgPWa3rGBRjsyKxCfAcspz0IZtwA7f2UHXPycpiT
// SIG // sRHWj++O6qz0llEM0I3fMVJPXq99p0p15k7OK58pK+fP
// SIG // XZ8Dfm4neyOtYilmr5LoLM69nH/czRc8Q8IJP3VFx57k
// SIG // 12B/l8bkqsdp9fzNdKwssEjBUE5wVh61NdOOvrHtrL3+
// SIG // DOyFfdW7hWZEGV2fk+uCumOe03xh/8gb2SNYfzCjZqE5
// SIG // Jl6SwzzLNzL69aON3NWwo+klNlXXgfowggPEMIIDLaAD
// SIG // AgECAhBHvxmV341SRkP3221IDTGkMA0GCSqGSIb3DQEB
// SIG // BQUAMIGLMQswCQYDVQQGEwJaQTEVMBMGA1UECBMMV2Vz
// SIG // dGVybiBDYXBlMRQwEgYDVQQHEwtEdXJiYW52aWxsZTEP
// SIG // MA0GA1UEChMGVGhhd3RlMR0wGwYDVQQLExRUaGF3dGUg
// SIG // Q2VydGlmaWNhdGlvbjEfMB0GA1UEAxMWVGhhd3RlIFRp
// SIG // bWVzdGFtcGluZyBDQTAeFw0wMzEyMDQwMDAwMDBaFw0x
// SIG // MzEyMDMyMzU5NTlaMFMxCzAJBgNVBAYTAlVTMRcwFQYD
// SIG // VQQKEw5WZXJpU2lnbiwgSW5jLjErMCkGA1UEAxMiVmVy
// SIG // aVNpZ24gVGltZSBTdGFtcGluZyBTZXJ2aWNlcyBDQTCC
// SIG // ASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAKnK
// SIG // sqTMzSCvCn2JrId18LRO8d/BD79nYb2jZBzau/nKM6uE
// SIG // MIlYfozba902ng+/0ex48nemfm88v5OvDbpo9GyUyr1S
// SIG // LatIPfW21V1fGwKf+i9rHqT3o5qmGsgC4X9MUuMOYOxA
// SIG // HH65Dd4/x7Tfh71femoxLgOZgROoRyDOMXMNVy3NeDQz
// SIG // lRKZErneaC+q5uPCiowqw4shh2a9g1hXb3W/PKomh13K
// SIG // EBU8n4TqVMEKbsT+xUrduQcRlyJ82z4n0R547J8xyfHm
// SIG // IhnbxLNHQ5oaX6AekORe9e588X2rYgGP9U0L3tAiVqiV
// SIG // za6Idq7uug3z5E3ZoPtooK4UO7OHwbsCAwEAAaOB2zCB
// SIG // 2DA0BggrBgEFBQcBAQQoMCYwJAYIKwYBBQUHMAGGGGh0
// SIG // dHA6Ly9vY3NwLnZlcmlzaWduLmNvbTASBgNVHRMBAf8E
// SIG // CDAGAQH/AgEAMEEGA1UdHwQ6MDgwNqA0oDKGMGh0dHA6
// SIG // Ly9jcmwudmVyaXNpZ24uY29tL1RoYXd0ZVRpbWVzdGFt
// SIG // cGluZ0NBLmNybDATBgNVHSUEDDAKBggrBgEFBQcDCDAO
// SIG // BgNVHQ8BAf8EBAMCAQYwJAYDVR0RBB0wG6QZMBcxFTAT
// SIG // BgNVBAMTDFRTQTIwNDgtMS01MzANBgkqhkiG9w0BAQUF
// SIG // AAOBgQBKa/nqWMJEHDGJeZkrlr+CrAHWHEzNsIpYbt8I
// SIG // KaNeyMqTE+cEUg3vRycvADiw5MmTTprUImIV9z83IU9w
// SIG // MYDxiziHs+jolwD+z1WWTiTSqSdOeq63YUHzKs7nydle
// SIG // 3bsrhT61nbXZ4Vf/vrTFfvXPDJ7wl/4r0ztSGxs4J/c/
// SIG // SjCCBCgwggMQoAMCAQICCwQAAAAAAS9O4TVcMA0GCSqG
// SIG // SIb3DQEBBQUAMFcxCzAJBgNVBAYTAkJFMRkwFwYDVQQK
// SIG // ExBHbG9iYWxTaWduIG52LXNhMRAwDgYDVQQLEwdSb290
// SIG // IENBMRswGQYDVQQDExJHbG9iYWxTaWduIFJvb3QgQ0Ew
// SIG // HhcNMTEwNDEzMTAwMDAwWhcNMTkwNDEzMTAwMDAwWjBR
// SIG // MQswCQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2ln
// SIG // biBudi1zYTEnMCUGA1UEAxMeR2xvYmFsU2lnbiBDb2Rl
// SIG // U2lnbmluZyBDQSAtIEcyMIIBIjANBgkqhkiG9w0BAQEF
// SIG // AAOCAQ8AMIIBCgKCAQEAsk8U5xC+1yZyqzaX71O/QoRe
// SIG // WNGKKPxDRm9+KERQC3VdANc8CkSeIGqk90VKN2Cjbj8S
// SIG // +m36tkbDaqO4DCcoAlco0VD3YTlVuMPhJYZSPL8FHdez
// SIG // mviaJDFJ1aKp4tORqz48c+/2KfHINdAwe39OkqUGj4fi
// SIG // zvXBY2asGGkqwV67Wuhulf87gGKdmcfHL2bV/WIaglVa
// SIG // xvpAd47JMDwb8PI1uGxZnP3p1sq0QB73BMrRZ6l046UI
// SIG // VNmDNTuOjCMMdbbehkqeGj4KUEk4nNKokL+Y+siMKycR
// SIG // fir7zt6prjiTIvqm7PtcYXbDRNbMDH4vbQaAonRAu7cf
// SIG // 9DvXc1Qf8wIDAQABo4H6MIH3MA4GA1UdDwEB/wQEAwIB
// SIG // BjASBgNVHRMBAf8ECDAGAQH/AgEAMB0GA1UdDgQWBBQI
// SIG // bti2nIq/7T7Xw3RdzIAfqC9QejBHBgNVHSAEQDA+MDwG
// SIG // BFUdIAAwNDAyBggrBgEFBQcCARYmaHR0cHM6Ly93d3cu
// SIG // Z2xvYmFsc2lnbi5jb20vcmVwb3NpdG9yeS8wMwYDVR0f
// SIG // BCwwKjAooCagJIYiaHR0cDovL2NybC5nbG9iYWxzaWdu
// SIG // Lm5ldC9yb290LmNybDATBgNVHSUEDDAKBggrBgEFBQcD
// SIG // AzAfBgNVHSMEGDAWgBRge2YaRQ2XyolQL30EzTSo//z9
// SIG // SzANBgkqhkiG9w0BAQUFAAOCAQEAIlzF3T30C3DY4/Xn
// SIG // xY4JAbuxljZcWgetx6hESVEleq4NpBk7kpzPuUImuzts
// SIG // l+fHzhFtaJHajW3xU01UOIxh88iCdmm+gTILMcNsyZ4g
// SIG // Clgv8Ej+fkgHqtdDWJRzVAQxqXgNO4ywcME9fte9LyrD
// SIG // 4vWPDJDca6XIvmheXW34eNK+SZUeFXgIkfs0yL6Erbzg
// SIG // xt0Y2/PK8HvCFDwYuAO6lT4hHj9gaXp/agOejUr58Cgs
// SIG // MIRe7CZyQrFty2TDEozWhEtnQXyxAxd4CeOtqLaWLaR+
// SIG // gANPiPfBa1pGFc0sGYvYcJzlLUmIYHKopBlScENe2tZG
// SIG // A7BoDiTvSvYLJSTvJDCCBKowggOSoAMCAQICEhEhd0T0
// SIG // c0IDCG56KyMS/hIl1TANBgkqhkiG9w0BAQUFADBRMQsw
// SIG // CQYDVQQGEwJCRTEZMBcGA1UEChMQR2xvYmFsU2lnbiBu
// SIG // di1zYTEnMCUGA1UEAxMeR2xvYmFsU2lnbiBDb2RlU2ln
// SIG // bmluZyBDQSAtIEcyMB4XDTEyMDcyMDE1MjY0MloXDTE1
// SIG // MDcyMTE1MjY0MlowdzELMAkGA1UEBhMCU0sxEzARBgNV
// SIG // BAgTCkJyYXRpc2xhdmExEzARBgNVBAcTCkJyYXRpc2xh
// SIG // dmExHjAcBgNVBAoTFU1hcnRpbiBNYWxpayAtIFJFQUxp
// SIG // WDEeMBwGA1UEAxMVTWFydGluIE1hbGlrIC0gUkVBTGlY
// SIG // MIIBIjANBgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEA
// SIG // 3FrGxIGVpRZxQbvame34ZwMZax+fSFQF8o/AKnoT/rwu
// SIG // gDZ+Tthyp2mz7e9vBL+pkVDFw3DSNEev+7WBb9+7Tf2m
// SIG // WipPP6bjGmKfs0fZrFtTrD1JGowVbBsaJCtOqU++IBvV
// SIG // IYhpY4nMk+aGPC2/wO5qLX4UdElm1nUEE/pSZllhakIi
// SIG // M8gYmWk2RoQV0JMubwL1pfWDx7SYWqA85j1wOkEKiPCC
// SIG // hxvn7G4Ebq8HCrQqkV2hUM8JAznHP0jBTVy3FiT0Duob
// SIG // C4SAHqHMQBdhyJL4ZrDrzbWFIBTZiIbw85vYwMDiNVq9
// SIG // A9z6vusjTLDfwdwuqibmBA3VrQJ04VKMmwIDAQABo4IB
// SIG // VDCCAVAwDgYDVR0PAQH/BAQDAgeAMEwGA1UdIARFMEMw
// SIG // QQYJKwYBBAGgMgEyMDQwMgYIKwYBBQUHAgEWJmh0dHBz
// SIG // Oi8vd3d3Lmdsb2JhbHNpZ24uY29tL3JlcG9zaXRvcnkv
// SIG // MAkGA1UdEwQCMAAwEwYDVR0lBAwwCgYIKwYBBQUHAwMw
// SIG // PgYDVR0fBDcwNTAzoDGgL4YtaHR0cDovL2NybC5nbG9i
// SIG // YWxzaWduLmNvbS9ncy9nc2NvZGVzaWduZzIuY3JsMFAG
// SIG // CCsGAQUFBwEBBEQwQjBABggrBgEFBQcwAoY0aHR0cDov
// SIG // L3NlY3VyZS5nbG9iYWxzaWduLmNvbS9jYWNlcnQvZ3Nj
// SIG // b2Rlc2lnbmcyLmNydDAdBgNVHQ4EFgQUEDhRUZO9/J8Y
// SIG // VQwfX+nMHBUewOcwHwYDVR0jBBgwFoAUCG7YtpyKv+0+
// SIG // 18N0XcyAH6gvUHowDQYJKoZIhvcNAQEFBQADggEBAId+
// SIG // IzJGAm8J7AN6fv6ESIz2P+J56fvF6y3kAr+YgAVCrV/N
// SIG // k0SfnrbEXMdWq8iC0QQZ9oGBL2OjufDDPExrEB9kmi8v
// SIG // qrQtsncfrxXkd6mBFKETVCC0mQuSd0pWqoITCkq5i2Qp
// SIG // iy2BVlUXLE8khHsGeTKR7lDGpTi031IO+oxzKwk40vBY
// SIG // QJvcKy+XftkRS/tvy9l7ayukRU1BZb6/MEB1q/e4m93f
// SIG // mHPA3P3lQEeBawEEmj0W3WEX8R/6qtmYdF9ORD/HgnK6
// SIG // osJcpLyUzyY14Y6wM8fiu6ncKHZOfdLp7PIKoTIW6G6N
// SIG // 453UHTlybnD11uZRJH5jDGQuYwRdvyQxggOiMIIDngIB
// SIG // ATBnMFExCzAJBgNVBAYTAkJFMRkwFwYDVQQKExBHbG9i
// SIG // YWxTaWduIG52LXNhMScwJQYDVQQDEx5HbG9iYWxTaWdu
// SIG // IENvZGVTaWduaW5nIENBIC0gRzICEhEhd0T0c0IDCG56
// SIG // KyMS/hIl1TAJBgUrDgMCGgUAoIGOMBkGCSqGSIb3DQEJ
// SIG // AzEMBgorBgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAM
// SIG // BgorBgEEAYI3AgEVMCMGCSqGSIb3DQEJBDEWBBSnPIyE
// SIG // zjPUxIhafUj6y7hlptQHqjAuBgorBgEEAYI3AgEMMSAw
// SIG // HqAcgBoASABXAGkATgBGAE8ATQBvAG4AaQB0AG8AcjAN
// SIG // BgkqhkiG9w0BAQEFAASCAQBx1HC4KoeHtjp7suUghImv
// SIG // ShXmXUz4Fiy8FRl6JIbKEIXqzlVhUeHsK5Cdjv+mHo3C
// SIG // cm2gbEIWc62Hpl4LocooSl9G46hMG/SZ4lMkIP9WBBuI
// SIG // QAreRpi5HY2w0/Y/q26wTdw5JxvfnVNQQIrHQLoXvoxc
// SIG // sHcKOgDVzuSR8EtVwsQtSQ3G8ptEbZ/HsU7KKEqyegMI
// SIG // zr7yU34f9JUeO+P1qQTP8su41sJJw918A+4T7IeHN+Et
// SIG // 58tDk2BMRGyJRcnhDTcC/pT0eXqQE1CHRfImQIMfEEVm
// SIG // lwlx/CH6tAjr4ZO1R82rQx9Jfb8jZWxm6fSHmgShSDCi
// SIG // dcYdBJV1mV1ZoYIBfzCCAXsGCSqGSIb3DQEJBjGCAWww
// SIG // ggFoAgEBMGcwUzELMAkGA1UEBhMCVVMxFzAVBgNVBAoT
// SIG // DlZlcmlTaWduLCBJbmMuMSswKQYDVQQDEyJWZXJpU2ln
// SIG // biBUaW1lIFN0YW1waW5nIFNlcnZpY2VzIENBAhB5oqWF
// SIG // +dEVQhPZuD72to3tMAkGBSsOAwIaBQCgXTAYBgkqhkiG
// SIG // 9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEP
// SIG // Fw0xMjA4MDQxODM2MjZaMCMGCSqGSIb3DQEJBDEWBBQX
// SIG // yJ+jCMWlPY7EaLjPK3BCmjinejANBgkqhkiG9w0BAQEF
// SIG // AASBgBANdRYz+viH5Iw4ykGuad1TNNdepjua6B7yyyCn
// SIG // lGG6Kd3xUs6gCoX7lqlJyC2ZGarNfbiwkKj3h43SDgFv
// SIG // qYKWC2067NoCfBwx8YeCiXpNGDDmag9d34+cNoYID0Jl
// SIG // 85ohTjVyONJJ0LVbGUbnnwh//UJi8SKdZjQk1vPWtfYm
// SIG // End signature block
